@@ -21,6 +21,7 @@ import org.jspecify.annotations.Nullable;
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileNotFoundException;
+import java.io.FileReader;
 import java.io.IOException;
 import java.io.Reader;
 import java.io.UncheckedIOException;
@@ -28,9 +29,6 @@ import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import static java.nio.charset.StandardCharsets.UTF_8;
-import static java.nio.file.Files.newBufferedReader;
-
 /**
  * Preprocess an Antlr grammar file so that dependencies between grammars can be properly determined such that they can
  * be processed for generation in proper order later.
@@ -73,9 +71,11 @@ public class MetadataExtracter {
     }
 
     @Nullable
-    private String getPackageName(File grammarFileFile) {
+    @SuppressWarnings("DefaultCharset")
+    private String getPackageName(File grammarFile) {
         try {
-            return getPackageName(newBufferedReader(grammarFileFile.toPath(), UTF_8));
+            // Note: source files can have non-UTF8 encoding. FileReader uses default Charset and also handles invalid characters.
+            return getPackageName(new FileReader(grammarFile));
         } catch (IOException e) {
             throw new UncheckedIOException("Cannot read antlr grammar file", e);
         }
