@@ -23,13 +23,12 @@ import org.gradle.api.tasks.VerificationException;
 import org.gradle.execution.plan.edges.DependencyNodesSet;
 import org.gradle.execution.plan.edges.DependentNodesSet;
 import org.gradle.internal.resources.ResourceLock;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nullable;
 import javax.annotation.OverridingMethodsMustInvokeSuper;
 import java.util.Collections;
 import java.util.List;
 import java.util.Set;
-import java.util.SortedSet;
 import java.util.function.Consumer;
 
 /**
@@ -70,7 +69,8 @@ public abstract class Node {
     private int index;
     private DependencyNodesSet dependencyNodes = DependencyNodesSet.EMPTY;
     private DependentNodesSet dependentNodes = DependentNodesSet.EMPTY;
-    private final MutationInfo mutationInfo = new MutationInfo();
+    private MutationInfo mutationInfo = MutationInfo.EMPTY;
+    private final ConsumerState consumerState = new ConsumerState();
     private NodeGroup group = NodeGroup.DEFAULT_GROUP;
 
     @VisibleForTesting
@@ -171,12 +171,17 @@ public abstract class Node {
         }
     }
 
+    /**
+     * Maybe update the group for this node when its dependencies are in groups with higher ordinal.
+     *
+     * That way we can ensure that the node is executed after all its dependencies, and we get rid of potential cycles.
+     */
     public void maybeUpdateOrdinalGroup() {
         OrdinalGroup ordinal = getGroup().asOrdinal();
         OrdinalGroup newOrdinal = ordinal;
         for (Node successor : getHardSuccessors()) {
             OrdinalGroup successorOrdinal = successor.getGroup().asOrdinal();
-            if (successorOrdinal != null && (ordinal == null || successorOrdinal.getOrdinal() > ordinal.getOrdinal())) {
+            if (successorOrdinal != null && (newOrdinal == null || successorOrdinal.getOrdinal() > newOrdinal.getOrdinal())) {
                 newOrdinal = successorOrdinal;
             }
         }
@@ -367,7 +372,7 @@ public abstract class Node {
         return this.executionFailure;
     }
 
-    public SortedSet<Node> getDependencyPredecessors() {
+    public Set<Node> getDependencyPredecessors() {
         return dependentNodes.getDependencyPredecessors();
     }
 
@@ -382,7 +387,7 @@ public abstract class Node {
 
     void addDependencyPredecessor(Node fromNode) {
         dependentNodes = dependentNodes.addDependencyPredecessors(fromNode);
-        mutationInfo.addConsumer(fromNode);
+        consumerState.addConsumer(fromNode);
     }
 
     void addMustPredecessor(TaskNode fromNode) {
@@ -493,8 +498,7 @@ public abstract class Node {
         }
     }
 
-    @Nullable
-    protected Node.ExecutionState getInitialState() {
+    protected Node.@Nullable ExecutionState getInitialState() {
         return null;
     }
 
@@ -542,7 +546,7 @@ public abstract class Node {
         dependencyNodes.getDependencySuccessors().forEach(visitor);
     }
 
-    public SortedSet<Node> getFinalizers() {
+    public Set<Node> getFinalizers() {
         return dependentNodes.getFinalizers();
     }
 
@@ -584,10 +588,18 @@ public abstract class Node {
     public void visitPostExecutionNodes(Consumer<? super Node> visitor) {
     }
 
+    public void mutationsResolved(MutationInfo mutationInfo) {
+        this.mutationInfo = mutationInfo;
+    }
+
     public MutationInfo getMutationInfo() {
         return mutationInfo;
     }
 
+    public ConsumerState getConsumerState() {
+        return consumerState;
+    }
+
     public boolean isPublicNode() {
         return false;
     }
