@@ -15,8 +15,8 @@
  */
 package org.gradle.api.internal.tasks.testing.report;
 
-import org.gradle.api.NonNullApi;
 import org.gradle.api.internal.tasks.testing.junit.result.TestResultsProvider;
+import org.jspecify.annotations.NullMarked;
 
 import java.io.File;
 
@@ -26,10 +26,14 @@ import java.io.File;
  * <strong>This is an internal API</strong>
  * <p>
  * This interface is currently implemented and required by cashapp/paparazzi
+ * <p>
+ * This interface is deprecated.
+ * </p>
  *
  * @see <a href="https://github.com/cashapp/paparazzi/blob/bf86c4f0ffe3da60cf1e0b4988c32e1c027f07a3/paparazzi-gradle-plugin/src/main/java/app/cash/paparazzi/gradle/PaparazziPlugin.kt#L290C40-L296">link</a>
  */
-@NonNullApi
+@NullMarked
+// This is not actually @Deprecated because it causes an unsuppressable warning where imported
 public interface TestReporter {
     void generateReport(TestResultsProvider testResultsProvider, File reportDir);
 }
